@@ -1,17 +1,21 @@
 /*
 	Description: file history annotation
 
-	Author: Marco Costalba (C) 2005-2006
+	Author: Marco Costalba (C) 2005-2007
 
 	Copyright: See COPYING file that comes with this distribution
 
 */
-#include <qapplication.h>
+#include <QApplication>
+#include <QTimer>
 #include "git.h"
+#include "myprocess.h"
 #include "annotate.h"
 
 #define MAX_AUTHOR_LEN 16
 
+using namespace QGit;
+
 Annotate::Annotate(Git* parent, QObject* guiObj) : QObject(parent) {
 
 	EM_INIT(exAnnCanceled, "Canceling annotation");
@@ -19,56 +23,32 @@ Annotate::Annotate(Git* parent, QObject* guiObj) : QObject(parent) {
 	git = parent;
 	gui = guiObj;
 	cancelingAnnotate = annotateRunning = annotateActivity = false;
-	valid = canceled = false;
+	valid = canceled = isError = false;
 
-	patchProcBuf.reserve(1000000); // avoid repeated reallocation, will be big!
-
-	processingTime.start();
-
-	connect(&patchProc, SIGNAL(readyReadStdout()), this, SLOT(on_patchProc_readFromStdout()));
-	connect(&patchProc, SIGNAL(processExited()), this, SLOT(on_patchProc_processExited()));
+	connect(this, SIGNAL(annotateReady(Annotate*, bool, const QString&)),
+	        git, SIGNAL(annotateReady(Annotate*, bool, const QString&)));
 }
 
-const FileAnnotation* Annotate::lookupAnnotation(SCRef sha, SCRef fn) {
+const FileAnnotation* Annotate::lookupAnnotation(SCRef sha) {
 
-	if (!valid || fileName != fn)
+	if (!valid || sha.isEmpty())
 		return NULL;
 
-	AnnotateHistory::const_iterator it = ah.find(sha);
+	AnnotateHistory::const_iterator it = ah.constFind(toTempSha(sha));
 	if (it != ah.constEnd())
-		return &(it.data());
+		return &(it.value());
 
 	// ok, we are not lucky. Check for an ancestor before to give up
 	int shaIdx;
-	const QString ancestorSha = getAncestor(sha, fileName, &shaIdx);
+	const QString ancestorSha = getAncestor(sha, &shaIdx);
 	if (!ancestorSha.isEmpty()) {
-		it = ah.find(ancestorSha);
+		it = ah.constFind(toTempSha(ancestorSha));
 		if (it != ah.constEnd())
-			return &(it.data());
+			return &(it.value());
 	}
 	return NULL;
 }
 
-bool Annotate::startPatchProc(SCRef buf, SCRef fileName) {
-
-	QString cmd("git diff-tree -r -m --patch-with-raw --no-commit-id --stdin --");
-	QStringList args(QStringList::split(' ', cmd));
-	args.append(fileName); // handle file name with spaces case
-	patchProc.setArguments(args);
-	patchProc.setWorkingDirectory(git->workDir);
-	patchProcBuf = "";
-	return patchProc.launch(buf);
-}
-
-void Annotate::on_patchProc_readFromStdout() {
-
-	const QString tmp(patchProc.readStdout());
-	annFilesNum += tmp.contains("diff --git ");
-	if (annFilesNum > (int)histRevOrder.count())
-		annFilesNum = histRevOrder.count();
-	patchProcBuf.append(tmp);
-}
-
 void Annotate::deleteWhenDone() {
 
 	if (!EM_IS_PENDING(exAnnCanceled))
@@ -77,9 +57,6 @@ void Annotate::deleteWhenDone() {
 	if (annotateRunning)
 		cancelingAnnotate = true;
 
-	if (patchProc.isRunning())
-		patchProc.tryTerminate();
-
 	on_deleteWhenDone();
 }
 
@@ -91,19 +68,10 @@ void Annotate::on_deleteWhenDone() {
 		QTimer::singleShot(20, this, SLOT(on_deleteWhenDone()));
 }
 
-void Annotate::on_progressTimer_timeout() {
-
-	if (!cancelingAnnotate && !isError) {
-		const QString n(QString::number(annFilesNum));
-		QApplication::postEvent(gui, new AnnotateProgressEvent(n));
-	}
-}
-
 bool Annotate::start(const FileHistory* _fh) {
 
 	// could change during annotation, so save them
 	fh = _fh;
-	fileName = fh->fileName;
 	histRevOrder = fh->revOrder;
 
 	if (histRevOrder.isEmpty()) {
@@ -116,136 +84,95 @@ bool Annotate::start(const FileHistory* _fh) {
 	annFilesNum = 0;
 	annId = histRevOrder.count();
 	annNumLen = QString::number(histRevOrder.count()).length();
-	StrVect::const_iterator it(histRevOrder.constBegin());
+	ShaVect::const_iterator it(histRevOrder.constBegin());
 	do
 		ah.insert(*it, FileAnnotation(annId--));
 	while (++it != histRevOrder.constEnd());
 
-	// annotation is split in two parts.
-	// first we get the list of sha's to feed git diff-tree.
-	// This is very fast.
-	isError = false;
-	annotateFileHistory(fileName, true);
-
-	if (isError || cancelingAnnotate) {
-		slotComputeDiffs(); // clean-up in case of error/canceling
-		return false;
-	}
-	connect(&progressTimer, SIGNAL(timeout()), this, SLOT(on_progressTimer_timeout()));
-	progressTimer.start(500);
-
-	// now we get the patches with an async call. This is the slowest part.
-	return startPatchProc(patchScript, fileName);
-}
-
-void Annotate::on_patchProc_processExited() {
-
-	// start computing diffs only on return from event handler
-	QTimer::singleShot(1, this, SLOT(slotComputeDiffs()));
-	progressTimer.stop();
+	// annotating the file history could be time consuming,
+	// so return now and use a timer to start annotation
+	QTimer::singleShot(100, this, SLOT(slotComputeDiffs()));
+	return true;
 }
 
 void Annotate::slotComputeDiffs() {
 
-	// when all the patches are loaded we compute the annotation.
-	// this part is normally faster then getting the patches.
-	if (!cancelingAnnotate) {
+	processingTime.start();
 
-		diffMap.clear();
-		AnnotateHistory::iterator it(ah.begin());
-		do
-			(*it).isValid = false; // reset flags
-		while (++it != ah.end());
+	if (!cancelingAnnotate)
+		annotateFileHistory(); // now could call Qt event loop
 
-		// remove first 'next patch' marker
-		int first = patchProcBuf.find(':');
-		if (first != -1) {
-			nextFileSha = patchProcBuf.mid(first + 56, 40);
-			patchProcBuf.remove(0, first + 100);
-		}
-		annotateFileHistory(fileName, false); // now could call Qt event loop
-	}
 	valid = !(isError || cancelingAnnotate);
 	canceled = cancelingAnnotate;
 	cancelingAnnotate = annotateRunning = false;
 	if (canceled)
 		deleteWhenDone();
-	else
-		git->annotateExited(this);
-
-//	StrVect::const_iterator it(histRevOrder.constBegin());
-//	do {
-//		dbg(*it); dbg(ah[*it].fileSha);
-//	} while (++it != histRevOrder.constEnd());
+	else {
+		QString msg("%1 %2");
+		msg = msg.arg(ah.count()).arg(processingTime.elapsed());
+		emit annotateReady(this, valid, msg);
+	}
 }
 
-void Annotate::annotateFileHistory(SCRef fileName, bool buildPatchScript) {
+void Annotate::annotateFileHistory() {
 
 	// sweep from the oldest to newest so that parent
 	// annotations are calculated before children
-	StrVect::const_iterator it(histRevOrder.constEnd());
+	ShaVect::const_iterator it(histRevOrder.constEnd());
 	do {
 		--it;
-		doAnnotate(fileName, *it, buildPatchScript);
+		doAnnotate(*it);
 	} while (it != histRevOrder.constBegin() && !isError && !cancelingAnnotate);
 }
 
-void Annotate::doAnnotate(SCRef fileName, SCRef sha, bool buildPatchScript) {
+void Annotate::doAnnotate(const ShaString& ss) {
 // all the parents annotations must be valid here
 
+	const QString sha(ss);
 	FileAnnotation* fa = getFileAnnotation(sha);
 	if (fa == NULL || fa->isValid || isError || cancelingAnnotate)
 		return;
 
-	const Rev* r = git->revLookup(sha, fh); // historyRevs
+	const Rev* r = git->revLookup(ss, fh); // historyRevs
 	if (r == NULL) {
 		dbp("ASSERT doAnnotate: no revision %1", sha);
 		isError = true;
 		return;
 	}
+	const QString& diff(getPatch(sha)); // set FileAnnotation::fileSha
 	if (r->parentsCount() == 0) { // initial revision
-		if (!buildPatchScript)
-			setInitialAnnotation(fileName, sha, fa); // calls Qt event loop
+		setInitialAnnotation(ah[ss].fileSha, fa); // calls Qt event loop
 		fa->isValid = true;
 		return;
 	}
 	// now create a new annotation from first parent diffs
-	const QStringList parents(r->parents());
+	const QStringList& parents(r->parents());
 	const QString& parSha = parents.first();
 	FileAnnotation* pa = getFileAnnotation(parSha);
 
-	if (!(pa && pa->isValid)) {
+	if (!pa || !pa->isValid) {
 		dbp("ASSERT in doAnnotate: annotation for %1 not valid", parSha);
 		isError = true;
 		return;
 	}
-	if (buildPatchScript) // just prepare patch script
-		updatePatchScript(sha, parSha);
-	else {
-		const QString diff(getNextPatch(patchProcBuf, fileName, sha));
-		const QString author(setupAuthor(r->author(), fa->annId));
-		setAnnotation(diff, author, pa->lines, fa->lines);
-	}
+	const QString& author(setupAuthor(r->author(), fa->annId));
+	setAnnotation(diff, author, pa->lines, fa->lines);
+
 	// then add other parents diff if any
 	QStringList::const_iterator it(parents.constBegin());
 	++it;
+	int parentNum = 1;
 	while (it != parents.constEnd()) {
 
 		FileAnnotation* pa = getFileAnnotation(*it);
-
-		if (buildPatchScript) { // just prepare patch script
-			updatePatchScript(sha, *it);
-			++it;
-			continue;
-		}
-		const QString diff(getNextPatch(patchProcBuf, fileName, sha));
+		const QString& diff(getPatch(sha, parentNum++));
 		QStringList tmpAnn;
 		setAnnotation(diff, "Merge", pa->lines, tmpAnn);
 
 		// the two annotations must be of the same length
 		if (fa->lines.count() != tmpAnn.count()) {
-			qDebug("ASSERT: merging annotations of different length\n"
-			       " merging %s in %s", (*it).latin1(), sha.latin1());
+			qDebug("ASSERT: merging annotations of different length\n merging "
+			       "%s in %s", (*it).toLatin1().constData(), sha.toLatin1().constData());
 			isError = true;
 			return;
 		}
@@ -259,7 +186,7 @@ void Annotate::doAnnotate(SCRef fileName, SCRef sha, bool buildPatchScript) {
 
 FileAnnotation* Annotate::getFileAnnotation(SCRef sha) {
 
-	AnnotateHistory::iterator it(ah.find(sha));
+	AnnotateHistory::iterator it(ah.find(toTempSha(sha)));
 	if (it == ah.end()) {
 		dbp("ASSERT getFileAnnotation: no revision %1", sha);
 		isError = true;
@@ -268,67 +195,73 @@ FileAnnotation* Annotate::getFileAnnotation(SCRef sha) {
 	return &(*it);
 }
 
-void Annotate::setInitialAnnotation(SCRef fileName, SCRef sha, FileAnnotation* fa) {
+void Annotate::setInitialAnnotation(SCRef fileSha, FileAnnotation* fa) {
 
-	QString fileSha;
 	QByteArray fileData;
-	git->getFile(fileName, sha, NULL, &fileData, &fileSha); // calls Qt event loop
+
+	// fh->fileNames() are in cronological order, so we need the last one
+	git->getFile(fileSha, NULL, &fileData, fh->fileNames().last()); // calls Qt event loop
 	if (cancelingAnnotate)
 		return;
 
-	if (fileSha.isEmpty()) {
-		dbp("ASSERT in setInitialAnnotation: empty file of initial rev %1", sha);
-		isError = true;
-		return;
-	}
-	ah[sha].fileSha = fileSha;
-	QString fileTxt(fileData);
-	int lineNum = fileTxt.contains('\n');
-	if (!fileTxt.endsWith("\n")) // No newline at end of file
+	int lineNum = fileData.count('\n');
+	if (!fileData.endsWith('\n') && !fileData.isEmpty()) // No newline at end of file
 		lineNum++;
 
-	fa->lines.insert(fa->lines.end(), lineNum, "");
+	const QString empty;
+	for (int i = 0; i < lineNum; i++)
+		fa->lines.append(empty);
 }
 
 const QString Annotate::setupAuthor(SCRef origAuthor, int annId) {
 
-	QString author(QString("%1.").arg(annId, annNumLen)); // first field is annotation id
-	QString tmp(origAuthor.section('<', 0, 0).stripWhiteSpace()); // strip e-mail address
+	QString tmp(origAuthor.section('<', 0, 0).trimmed()); // strip e-mail address
 	if (tmp.isEmpty()) { // probably only e-mail
 		tmp = origAuthor;
 		tmp.remove('<').remove('>');
-		tmp = tmp.stripWhiteSpace();
+		tmp = tmp.trimmed();
 		tmp.truncate(MAX_AUTHOR_LEN);
 	}
 	// shrink author name if necessary
 	if (tmp.length() > MAX_AUTHOR_LEN) {
-		SCRef firstName(tmp.section(' ', 0, 0));
-		SCRef surname(tmp.section(' ', 1));
+		SCRef firstName(tmp.section(' ', 0, 0).trimmed());
+		SCRef surname(tmp.section(' ', 1).trimmed());
 		if (!firstName.isEmpty() && !surname.isEmpty())
 			tmp = firstName.left(1) + ". " + surname;
+
 		tmp.truncate(MAX_AUTHOR_LEN);
 	}
-	author.append(tmp);
-	return author;
+	return QString("%1.%2").arg(annId, annNumLen).arg(tmp);
 }
 
 void Annotate::unify(SList dst, SCList src) {
 
-	QStringList::Iterator itd(dst.begin());
-	QStringList::const_iterator its(src.constBegin());
-	for ( ; itd != dst.end(); ++itd, ++its)
-		if (*itd == "Merge")
-			*itd = *its;
+	const QString m("Merge");
+	for (int i = 0; i < dst.size(); ++i) {
+		if (dst.at(i) == m)
+			dst[i] = src.at(i);
+	}
 }
 
-void Annotate::setAnnotation(SCRef diff, SCRef author, SCList prevAnn,
-                             QStringList& newAnn, int ofs) {
-	newAnn = prevAnn;
-	QStringList::iterator cur(newAnn.begin());
+bool Annotate::setAnnotation(SCRef diff, SCRef author, SCList prevAnn, SList newAnn, int ofs) {
+
+	newAnn.clear();
+	QStringList::const_iterator cur(prevAnn.constBegin());
 	QString line;
 	int idx = 0, num, lineNumStart, lineNumEnd;
+	int curLineNum = 1; // warning, starts from 1 instead of 0
+	bool inHeader = true;
+
 	while (getNextSection(diff, idx, line, "\n")) {
-		char firstChar = line[0].latin1();
+
+		char firstChar = line.at(0).toLatin1();
+
+		if (inHeader) {
+			if (firstChar == '@')
+				inHeader = false;
+			else
+				continue;
+		}
 		switch (firstChar) {
 		case '@':
 			// an unified diff fragment header has form '@@ -a,b +c,d @@'
@@ -336,48 +269,47 @@ void Annotate::setAnnotation(SCRef diff, SCRef author, SCList prevAnn,
 			// number of lines of the hunk, 'c' and 'd' are the same
 			// for new file. If the file does not have enough lines
 			// then also the form '@@ -a +c @@' is used.
-			lineNumStart = line.find('+') + 1;
-			lineNumEnd = line.find(',', lineNumStart);
+			if (ofs == 0)
+				lineNumStart = line.indexOf('-') + 1;
+			else
+			// in this case we are given diff fragments with
+			// faked small files that span the fragment plus
+			// some padding. So we use 'c' instead of 'a' to
+			// find the beginning of our patch in the faked file,
+			// this value will be offsetted by ofs later
+				lineNumStart = line.indexOf('+') + 1;
+
+			lineNumEnd = line.indexOf(',', lineNumStart);
 			if (lineNumEnd == -1) // small file case
-				lineNumEnd = line.find(' ', lineNumStart);
+				lineNumEnd = line.indexOf(' ', lineNumStart);
 
 			num = line.mid(lineNumStart, lineNumEnd - lineNumStart).toInt();
 			num -= ofs; // offset for range filter computation
 
 			// diff lines start from 1, 0 is empty file,
 			// instead QValueList::at() starts from 0
-			if (num <= 0) { // file is deleted
-				if (num < 0)
-					dbp("ASSERT processDiff: start line number is %1", num);
-				newAnn.clear();
-				return;
+			if (num < 0 || num > prevAnn.size()) {
+				dbp("ASSERT setAnnotation: start line number is %1", num);
+				isError = true;
+				return false;
+			}
+			for ( ; curLineNum < num; ++curLineNum) {
+				newAnn.append(*cur);
+				++cur;
 			}
-			cur = newAnn.at(num - 1);
 			break;
 		case '+':
-			if (cur != newAnn.end()) {
-				cur = newAnn.insert(cur, author);
-				++cur;
-			} else {
-				newAnn.append(author);
-				cur = newAnn.end();
-			}
+			newAnn.append(author);
 			break;
 		case '-':
-			if (!newAnn.isEmpty()) {
-				if (cur != newAnn.end())
-					cur = newAnn.remove(cur);
-				else {
-					dbp("ASSERT processDiff: remove end of "
-					    "file, diff is %1", diff);
-					isError = true;
-					return;
-				}
-			} else {
-				dbp("ASSERT processDiff: remove line from "
-				    "empty annotation, diff is %1", diff);
+			if (curLineNum > prevAnn.size()) {
+				dbp("ASSERT setAnnotation: remove end of "
+				    "file, diff is %1", diff);
 				isError = true;
-				return;
+				return false;
+			} else {
+				++cur;
+				++curLineNum;
 			}
 			break;
 		case '\\':
@@ -388,75 +320,59 @@ void Annotate::setAnnotation(SCRef diff, SCRef author, SCList prevAnn,
 
 			// fall through
 		default:
-			++cur;
+			if (curLineNum > prevAnn.size()) {
+				dbp("ASSERT setAnnotation: end of "
+				    "file reached, diff is %1", diff);
+				isError = true;
+				return false;
+			} else {
+				newAnn.append(*cur);
+				++cur;
+				++curLineNum;
+			}
 			break;
 		}
 	}
-}
-
-void Annotate::updatePatchScript(SCRef sha, SCRef par) {
-
-	if (sha == QGit::ZERO_SHA) // diff-tree --stdin doesn't work with working
-		return;            // dir patch will be directly fetched when needed
-
-	const QString runCmd(sha + " " + par);
-	patchScript.append(runCmd).append('\n');
-}
-
-const QString Annotate::getNextPatch(QString& patchFile, SCRef fileName, SCRef sha) {
-
-	if (sha == QGit::ZERO_SHA) {
-		// diff-tree --stdin doesn't work with working dir so get it
-		// directly. We don't need file sha info because is ZERO_SHA
-		QString runOutput;
-		QString runCmd("git diff-index -r -m -p HEAD -- " + QGit::QUOTE_CHAR +
-				fileName + QGit::QUOTE_CHAR);
-
-		git->run(runCmd, &runOutput);
-		if (cancelingAnnotate)
-			return "";
-
-		// restore removed head line of next patch
-		const QString nextHeader("\n:100644 100644 " + QGit::ZERO_SHA + ' '
-		                         + nextFileSha + " M\t" + fileName + '\n');
-		runOutput.append(nextHeader);
-		patchFile.prepend(runOutput);
-		nextFileSha = QGit::ZERO_SHA;
+	// copy the tail
+	for ( ; curLineNum <= prevAnn.size(); ++curLineNum) {
+		newAnn.append(*cur);
+		++cur;
 	}
-	// use patchProcBuf to get proper diff. Patches in patchProcBuf are
-	// correctly ordered, so take the first patch
-	ah[sha].fileSha = nextFileSha;
+	return true;
+}
 
-	bool noNewLine = (patchFile[0] == ':');
-	if (noNewLine)
-		dbp("WARNING: No newline at the end of %1 patch", sha);
+const QString Annotate::getPatch(SCRef sha, int parentNum) {
 
-	int end = (noNewLine) ? 0 : patchFile.find("\n:");
-	QString diff;
-	if (end != -1) {
-		diff = patchFile.left(end + 1);
-		nextFileSha = patchFile.mid(end + 57 - (int)noNewLine, 40);
-		patchFile.remove(0, end + 100); // the whole line until file name
-	} else
-		diff = patchFile;
+	QString mergeSha(sha);
+	if (parentNum)
+		mergeSha = QString::number(parentNum) + " m " + sha;
 
-	int start = diff.find('@');
-	// handle a possible file mode only change and remove header
-	diff = (start != -1) ? diff.mid(start) : "";
+	const Rev* r = git->revLookup(mergeSha, fh);
+	if (!r)
+		return QString();
 
-	int i = 0;
-	while (diffMap.contains(Key(sha, i)))
-		i++;
-	diffMap.insert(Key(sha, i), diff);
+	const QString diff(r->diff());
+
+	QVector<QByteArray> ba;
+	const ShaString& ss = toPersistentSha(sha, ba);
+
+	if (ah[ss].fileSha.isEmpty() && !parentNum) {
+
+		int idx = diff.indexOf("..");
+		if (idx != -1)
+			ah[ss].fileSha = diff.mid(idx + 2, 40);
+		else // file mode change only, same sha of parent
+			ah[ss].fileSha = ah[r->parent(0)].fileSha;
+	}
 	return diff;
 }
 
 bool Annotate::getNextSection(SCRef d, int& idx, QString& sec, SCRef target) {
 
-	if (idx >= (int)d.length())
+	if (idx >= d.length())
 		return false;
 
-	int newIdx = d.find(target, idx);
+	int newIdx = d.indexOf(target, idx);
 	if (newIdx == -1) // last section, take all
 		newIdx = d.length() - 1;
 
@@ -472,11 +388,11 @@ bool Annotate::getNextSection(SCRef d, int& idx, QString& sec, SCRef target) {
 
 bool Annotate::getRange(SCRef sha, RangeInfo* r) {
 
-	if (!rangeMap.contains(sha) || !valid || canceled) {
+	if (!ranges.contains(sha) || !valid || canceled) {
 		r->clear();
 		return false;
 	}
-	*r = rangeMap[sha]; // by copy
+	*r = ranges[sha]; // by copy
 	return true;
 }
 
@@ -503,7 +419,8 @@ void Annotate::updateCrossRanges(SCRef chunk, bool rev, int fileLen, int ofs, Ra
 	// because of the padding the file first line number will be
 	//
 	// fileFirstLineNr = newLineId - beforePadding = fileOffset + 1
-	QStringList beforeAnn, afterAnn;
+	QStringList beforeAnn;
+	QStringList afterAnn;
 	for (int lineNum = ofs + 1; lineNum <= ofs + fileLen; lineNum++)
 		beforeAnn.append(QString::number(lineNum));
 
@@ -515,8 +432,15 @@ void Annotate::updateCrossRanges(SCRef chunk, bool rev, int fileLen, int ofs, Ra
 	if (rev) {
 		// let's see what line number we have at given range interval limits.
 		// at() counts from 0.
-		QStringList::const_iterator itStart(afterAnn.at(r->start - ofs - 1));
-		QStringList::const_iterator itEnd(afterAnn.at(r->end - ofs - 1));
+		//QLinkedList<QString>::const_iterator itStart(afterAnn.at(r->start - ofs - 1));
+		//QLinkedList<QString>::const_iterator itEnd(afterAnn.at(r->end - ofs - 1));
+		QStringList::const_iterator itStart(afterAnn.begin());
+		for (int i = 0; i < r->start - ofs - 1; i++)
+			++itStart;
+
+		QStringList::const_iterator itEnd(afterAnn.begin());
+		for (int i = 0; i < r->end - ofs - 1; i++)
+			++itEnd;
 
 		bool leftExtended = (*itStart == fakedAuthor);
 		bool rightExtended = (*itEnd == fakedAuthor);
@@ -564,7 +488,7 @@ void Annotate::updateCrossRanges(SCRef chunk, bool rev, int fileLen, int ofs, Ra
 		QStringList::const_iterator itStart(afterAnn.constEnd());
 		QStringList::const_iterator itEnd(afterAnn.constEnd());
 
-		QStringList::const_iterator it(afterAnn.constBegin());
+		QStringList::const_iterator it(afterAnn.begin());
 		for (int lineNum = ofs + 1; it != afterAnn.constEnd(); ++lineNum, ++it) {
 
 			if (*it != fakedAuthor) {
@@ -631,12 +555,12 @@ void Annotate::updateRange(RangeInfo* r, SCRef diff, bool reverse) {
 		// number of lines of the hunk, 'c' and 'd' are the same
 		// for new file. If the file does not have enough lines
 		// then also the form '@@ -a +c @@' is used.
-		chunk = *chunkIt++;
-		int m = chunk.find('-');
-		int c1 = chunk.find(',', m);
-		int p = chunk.find('+', c1);
-		int c2 = chunk.find(',', p);
-		int e = chunk.find(' ', c2);
+		chunk  = *chunkIt++;
+		int m  = chunk.indexOf('-');
+		int c1 = chunk.indexOf(',', m);
+		int p  = chunk.indexOf('+', c1);
+		int c2 = chunk.indexOf(',', p);
+		int e  = chunk.indexOf(' ', c2);
 
 		int oldLineCnt = chunk.mid(c1 + 1, p - c1 - 2).toInt();
 		int newLineId = chunk.mid(p + 1, c2 - p - 1).toInt();
@@ -648,7 +572,7 @@ void Annotate::updateRange(RangeInfo* r, SCRef diff, bool reverse) {
 		int patchStart = newLineId;
 
 		// patch end depends only to lines count so is...
-		int patchEnd = patchStart + ((reverse) ? newLineCnt : oldLineCnt);
+		int patchEnd = patchStart + (reverse ? newLineCnt : oldLineCnt);
 		patchEnd--; // with 1 line patch patchStart == patchEnd
 
 		// case 1: patch range after our range
@@ -657,13 +581,13 @@ void Annotate::updateRange(RangeInfo* r, SCRef diff, bool reverse) {
 
 		// case 2: patch range before our range
 		if (patchEnd < r->start) {
-			r->start += ((reverse) ? -lineNumDiff : lineNumDiff);
-			r->end += ((reverse) ? -lineNumDiff : lineNumDiff);
+			r->start += (reverse ? -lineNumDiff : lineNumDiff);
+			r->end += (reverse ? -lineNumDiff : lineNumDiff);
 			continue;
 		}
 		// case 3: the patch is whole inside our range
 		if (patchStart >= r->start && patchEnd <= r->end) {
-			r->end += ((reverse) ? -lineNumDiff : lineNumDiff);
+			r->end += (reverse ? -lineNumDiff : lineNumDiff);
 			r->modified = true;
 			continue;
 		}
@@ -671,10 +595,10 @@ void Annotate::updateRange(RangeInfo* r, SCRef diff, bool reverse) {
 		// add padding so that resulting file is the UNION: selectRange U patchRange
 
 		// reverse independent
-		int beforePadding = (r->start > patchStart) ? 0 : patchStart - r->start;
+		int beforePadding = (r->start > patchStart ? 0 : patchStart - r->start);
 
 		// reverse dependent
-		int afterPadding = (patchEnd > r->end) ? 0 : r->end - patchEnd;
+		int afterPadding = (patchEnd > r->end ? 0 : r->end - patchEnd);
 
 		// file is the faked file on which we will apply the diff,
 		// so it is always the _old_ before the patch one.
@@ -696,7 +620,7 @@ void Annotate::updateRange(RangeInfo* r, SCRef diff, bool reverse) {
 	}
 }
 
-const QString Annotate::getAncestor(SCRef sha, SCRef fileName, int* shaIdx) {
+const QString Annotate::getAncestor(SCRef sha, int* shaIdx) {
 
 	QString fileSha;
 
@@ -704,7 +628,12 @@ const QString Annotate::getAncestor(SCRef sha, SCRef fileName, int* shaIdx) {
 		annotateActivity = true;
 		EM_REGISTER(exAnnCanceled);
 
-		fileSha = git->getFileSha(fileName, sha); // calls qApp->processEvents()
+		QStringList fn(fh->fileNames());
+		FOREACH_SL (it, fn) {
+			fileSha = git->getFileSha(*it, sha); // calls qApp->processEvents()
+			if (!fileSha.isEmpty())
+				break;
+		}
 		if (fileSha.isEmpty()) {
 			dbp("ASSERT in getAncestor: empty file from %1", sha);
 			return "";
@@ -731,7 +660,7 @@ const QString Annotate::getAncestor(SCRef sha, SCRef fileName, int* shaIdx) {
 	// modify only file mode but no content. From the point of view of code
 	// range filtering this is equivalent, so we don't care to find the correct
 	// ancestor, but just the first revision with the same file sha
-	for (*shaIdx = 0; *shaIdx < (int)histRevOrder.count(); (*shaIdx)++) {
+	for (*shaIdx = 0; *shaIdx < histRevOrder.count(); (*shaIdx)++) {
 
 		const FileAnnotation& fa(ah[histRevOrder[*shaIdx]]);
 		if (fa.fileSha == fileSha)
@@ -776,27 +705,33 @@ bool Annotate::isDescendant(SCRef sha, SCRef target) {
   Only checking for copies would fix the corner case, but implementation
   is too difficult, so better accept this design limitation for now.
 */
-const QString Annotate::computeRanges(SCRef sha, int rangeStart, int rangeEnd, SCRef target) {
+const QString Annotate::computeRanges(SCRef sha, int paraFrom, int paraTo, SCRef target) {
 
-	rangeMap.clear();
+	ranges.clear();
 
-	if (!valid || canceled) {
+	if (!valid || canceled || sha.isEmpty()) {
 		dbp("ASSERT in computeRanges: annotation from %1 not valid", sha);
 		return "";
 	}
+	// paragraphs start from 0 but ranges from 1
+	int rangeStart = paraFrom + 1;
+	int rangeEnd = paraTo + 1;
+
 	QString ancestor(sha);
 	int shaIdx;
-	for (shaIdx = 0; shaIdx < (int)histRevOrder.count(); shaIdx++)
-		if (histRevOrder[shaIdx] == sha)
+	QVector<QByteArray> ba;
+	const ShaString& ss = toPersistentSha(sha, ba);
+	for (shaIdx = 0; shaIdx < histRevOrder.count(); shaIdx++)
+		if (histRevOrder[shaIdx] == ss)
 			break;
 
-	if (shaIdx == (int)histRevOrder.count()) { // not in history, find an ancestor
-		ancestor = getAncestor(sha, fileName, &shaIdx);
+	if (shaIdx == histRevOrder.count()) { // not in history, find an ancestor
+		ancestor = getAncestor(sha, &shaIdx);
 		if (ancestor.isEmpty())
 			return "";
 	}
 	// insert starting one, always included by default, could be removed after
-	rangeMap.insert(ancestor, RangeInfo(rangeStart, rangeEnd, true));
+	ranges.insert(ancestor, RangeInfo(rangeStart, rangeEnd, true));
 
 	// check if target is a descendant, so to skip back history walking
 	bool isDirectDescendant = isDescendant(ancestor, target);
@@ -807,20 +742,21 @@ const QString Annotate::computeRanges(SCRef sha, int rangeStart, int rangeEnd, S
 	QString curRevSha(curRev->sha());
 	while (curRevSha != oldest && !isDirectDescendant) {
 
-		if (!diffMap.contains(Key(curRevSha, 0))) {
+		const QString& diff(getPatch(curRevSha));
+		if (diff.isEmpty()) {
 			if (curRev->parentsCount() == 0)  // is initial
 				break;
 
 			dbp("ASSERT in rangeFilter 1: diff for %1 not found", curRevSha);
 			return "";
 		}
-		RangeInfo r(rangeMap[curRevSha]);
-		updateRange(&r, diffMap[Key(curRevSha, 0)], true);
+		RangeInfo r(ranges[curRevSha]);
+		updateRange(&r, diff, true);
 
 		// special case for modified flag. Mark always the 'after patch' revision
 		// with modified flag, not the before patch. So we have to stick the flag
 		// to the newer revision.
-		rangeMap[curRevSha].modified = r.modified;
+		ranges[curRevSha].modified = r.modified;
 
 		// if the second revision does not modify the range then r.modified == false
 		// and the first revision range is created with modified == false, if the
@@ -836,7 +772,7 @@ const QString Annotate::computeRanges(SCRef sha, int rangeStart, int rangeEnd, S
 
 		curRev = git->revLookup(curRev->parent(0), fh);
 		curRevSha = curRev->sha();
-		rangeMap.insert(curRevSha, r);
+		ranges.insert(curRevSha, r);
 
 		if (curRevSha == target) // stop now, no need to continue
 			return ancestor;
@@ -851,7 +787,7 @@ const QString Annotate::computeRanges(SCRef sha, int rangeStart, int rangeEnd, S
 
 		SCRef sha(histRevOrder[shaIdx]);
 
-		if (!rangeMap.contains(sha)) {
+		if (!ranges.contains(sha)) {
 
 			curRev = git->revLookup(sha, fh);
 
@@ -860,16 +796,17 @@ const QString Annotate::computeRanges(SCRef sha, int rangeStart, int rangeEnd, S
 				// insert an empty range, the whole branch will be ignored.
 				// Merge of outside branches are very rare so this solution
 				// seems enough if we don't want to dive in (useless) complications.
-				rangeMap.insert(sha, RangeInfo());
+				ranges.insert(sha, RangeInfo());
 				continue;
 			}
-			if (!diffMap.contains(Key(sha, 0))) {
+			const QString& diff(getPatch(sha));
+			if (diff.isEmpty()) {
 				dbp("ASSERT in rangeFilter 2: diff for %1 not found", sha);
 				return "";
 			}
-			SCRef parSha(curRev->parent(0));
+			QString parSha(curRev->parent(0));
 
-			if (!rangeMap.contains(parSha)) {
+			if (!ranges.contains(parSha)) {
 
 				if (isDirectDescendant) // we must be in a parallel lane, no need
 					continue;       // to compute range info, simply go on
@@ -877,9 +814,9 @@ const QString Annotate::computeRanges(SCRef sha, int rangeStart, int rangeEnd, S
 				dbp("ASSERT in rangeFilter: range info for %1 not found", parSha);
 				return "";
 			}
-			RangeInfo r(rangeMap[parSha]);
-			updateRange(&r, diffMap[Key(sha, 0)], false);
-			rangeMap.insert(sha, r);
+			RangeInfo r(ranges[parSha]);
+			updateRange(&r, diff, false);
+			ranges.insert(sha, r);
 
 			if (sha == target) // stop now, no need to continue
 				return ancestor;
@@ -893,22 +830,20 @@ bool Annotate::seekPosition(int* paraFrom, int* paraTo, SCRef fromSha, SCRef toS
 	if ((*paraFrom == 0 && *paraTo == 0) || fromSha == toSha)
 		return true;
 
-	QMap<QString, RangeInfo> backup;
-	backup = rangeMap;  // QMap is implicitly shared
+	Ranges backup(ranges); // implicitly shared
 
-	// paragraphs start from 0 but ranges from 1
-	if (computeRanges(fromSha, *paraFrom + 1, *paraTo + 1, toSha).isEmpty())
+	if (computeRanges(fromSha, *paraFrom, *paraTo, toSha).isEmpty())
 		goto fail;
 
-	if (!rangeMap.contains(toSha))
+	if (!ranges.contains(toSha))
 		goto fail;
 
-	*paraFrom = rangeMap[toSha].start - 1;
-	*paraTo = rangeMap[toSha].end - 1;
-	rangeMap = backup;
+	*paraFrom = ranges[toSha].start - 1;
+	*paraTo = ranges[toSha].end - 1;
+	ranges = backup;
 	return true;
 
 fail:
-	rangeMap = backup;
+	ranges = backup;
 	return false;
 }
