@@ -1,5 +1,5 @@
 /*
-	Author: Marco Costalba (C) 2005-2006
+	Author: Marco Costalba (C) 2005-2007
 
 	Copyright: See COPYING file that comes with this distribution
 
@@ -7,14 +7,13 @@
 #ifndef COMMON_H
 #define COMMON_H
 
-#include <qstringlist.h>
-#include <qvaluevector.h>
-#include <qdict.h>
-#include <qmap.h>
-#include <qevent.h>
-#include <qcolor.h>
-#include <qfont.h>
-#include <qvariant.h>
+#include <QColor>
+#include <QEvent>
+#include <QFont>
+#include <QHash>
+#include <QLatin1String>
+#include <QVariant>
+#include <QVector>
 
 /*
    QVariant does not support size_t type used in Qt containers, this is
@@ -25,13 +24,14 @@
    Function _valueOf() is used by debug macros
 */
 template<typename T> inline const QString _valueOf(const T& x) { return QVariant(x).toString(); }
+template<> inline const QString _valueOf(const QStringList& x) { return x.join(" "); }
 inline const QString& _valueOf(const QString& x) { return x; }
 inline const QString  _valueOf(size_t x) { return QString::number((uint)x); }
 
 // some debug macros
-#define dbg(x)    qDebug(#x " is <%s>", _valueOf(x).latin1())
-#define dbs(x)    qDebug(_valueOf(x))
-#define dbp(s, x) qDebug(QString(s).arg(_valueOf(x)))
+#define dbg(x)    qDebug(#x " is <%s>", _valueOf(x).toLatin1().constData())
+#define dbs(x)    qDebug(_valueOf(x).toLatin1().constData())
+#define dbp(s, x) qDebug(QString(s).arg(_valueOf(x)).toLatin1().constData())
 #define db1       qDebug("Mark Nr. 1")
 #define db2       qDebug("Mark Nr. 2")
 #define db3       qDebug("Mark Nr. 3")
@@ -45,11 +45,21 @@ inline const QString  _valueOf(size_t x) { return QString::number((uint)x); }
 
 #define FOREACH_SL(i, c)    FOREACH(QStringList, i, c)
 
+class QDataStream;
+class QProcess;
+class QSplitter;
+class QWidget;
+class ShaString;
+
 // type shortcuts
-typedef const QString&        SCRef;
-typedef QStringList&          SList;
-typedef const QStringList&    SCList;
-typedef QValueVector<QString> StrVect;
+typedef const QString&              SCRef;
+typedef QStringList&                SList;
+typedef const QStringList&          SCList;
+typedef QVector<QString>            StrVect;
+typedef QVector<ShaString>          ShaVect;
+typedef QSet<QString>               ShaSet;
+
+uint qHash(const ShaString&); // optimized custom hash for sha strings
 
 namespace QGit {
 
@@ -125,6 +135,9 @@ namespace QGit {
 	                                        x == BOUNDARY_R || x == BOUNDARY_L); }
 	inline bool isMerge(int x) { return (x == MERGE_FORK || x == MERGE_FORK_R ||
 	                                     x == MERGE_FORK_L || isBoundary(x)); }
+	inline bool isActive(int x) { return (x == ACTIVE || x == INITIAL || x == BRANCH ||
+	                                      isMerge(x)); }
+
 	// custom events
 	enum EventType {
 		ERROR_EV      = 65432,
@@ -155,7 +168,7 @@ namespace QGit {
 	const int DEF_GRAPH_COL_WIDTH = 80;
 	const int DEF_LOG_COL_WIDTH   = 500;
 	const int DEF_AUTH_COL_WIDTH  = 230;
-	const int DEF_TIME_COL_WIDTH  = 130;
+	const int DEF_TIME_COL_WIDTH  = 160;
 
 	// colors
 	extern const QColor BROWN;
@@ -167,138 +180,183 @@ namespace QGit {
 	extern const QColor DARK_GREEN;
 
 	// initialized at startup according to system wide settings
-	extern QColor ODD_LINE_COL;
-	extern QColor EVEN_LINE_COL;
-	extern QFont  TYPE_WRITER_FONT;
+	extern QColor  ODD_LINE_COL;
+	extern QColor  EVEN_LINE_COL;
+	extern QFont   STD_FONT;
+	extern QFont   TYPE_WRITER_FONT;
+	extern QString GIT_DIR;
 
 	// patches drag and drop
 	extern const QString PATCHES_DIR;
 	extern const QString PATCHES_NAME;
 
 	// git index parameters
+	extern const QByteArray ZERO_SHA_BA;
+	extern const ShaString  ZERO_SHA_RAW;
+
 	extern const QString ZERO_SHA;
 	extern const QString CUSTOM_SHA;
 	extern const QString ALL_MERGE_FILES;
 
 	// settings keys
+	extern const QString ORG_KEY;
 	extern const QString APP_KEY;
-	extern const QString FP_DIR_KEY;
-	extern const QString FPATCH_ARGS_KEY;
+	extern const QString GIT_DIR_KEY;
+	extern const QString PATCH_DIR_KEY;
+	extern const QString FMT_P_OPT_KEY;
+	extern const QString AM_P_OPT_KEY;
+	extern const QString STD_FNT_KEY;
+	extern const QString TYPWRT_FNT_KEY;
 	extern const QString FLAGS_KEY;
+	extern const QString CON_GEOM_KEY;
 	extern const QString CMT_GEOM_KEY;
-	extern const QString CMT_SPLIT_KEY;
+	extern const QString MAIN_GEOM_KEY;
+	extern const QString REV_GEOM_KEY;
 	extern const QString CMT_TEMPL_KEY;
 	extern const QString CMT_ARGS_KEY;
+	extern const QString RANGE_FROM_KEY;
+	extern const QString RANGE_TO_KEY;
+	extern const QString RANGE_OPT_KEY;
 	extern const QString EX_KEY;
 	extern const QString EX_PER_DIR_KEY;
 	extern const QString EXT_DIFF_KEY;
 	extern const QString REC_REP_KEY;
-	extern const QString MCR_NAME_KEY;
-	extern const QString MCR_TEXT_KEY;
-	extern const QString MCR_LIST_KEY;
-	extern const QString FONT_KEY;
+	extern const QString ACT_LIST_KEY;
+	extern const QString ACT_GEOM_KEY;
+	extern const QString ACT_GROUP_KEY;
+	extern const QString ACT_TEXT_KEY;
+	extern const QString ACT_FLAGS_KEY;
 
 	// settings default values
-	extern const QString CMT_GEOM_DEF;
-	extern const QString CMT_SPLIT_DEF;
 	extern const QString CMT_TEMPL_DEF;
 	extern const QString EX_DEF;
 	extern const QString EX_PER_DIR_DEF;
 	extern const QString EXT_DIFF_DEF;
-	extern const QString MCR_NAME_DEF;
 
 	// settings booleans
 	enum FlagType {
-		// removed obsolete option
-		MCR_REFRESH_F   = 2,
-		NUMBERS_F       = 4,
-		// removed obsolete option
-		MCR_CMD_LINE_F  = 16,
-		DIFF_INDEX_F    = 32,
-		SIGN_PATCH_F    = 64,
-		SIGN_CMT_F      = 128,
-		VERIFY_CMT_F    = 256,
-		// removed obsolete option
-		REL_DATE_F      = 1024,
-		ALL_BRANCHES_F  = 2048,
-		WHOLE_HISTORY_F = 4096,
-		RANGE_SELECT_F  = 8192
+		MSG_ON_NEW_F    = 1 << 0,
+		ACT_REFRESH_F   = 1 << 1,
+		NUMBERS_F       = 1 << 2,
+		LOG_DIFF_TAB_F  = 1 << 3,
+		ACT_CMD_LINE_F  = 1 << 4,
+		DIFF_INDEX_F    = 1 << 5,
+		SIGN_PATCH_F    = 1 << 6,
+		SIGN_CMT_F      = 1 << 7,
+		VERIFY_CMT_F    = 1 << 8,
+		SMART_LBL_F     = 1 << 9,
+		REL_DATE_F      = 1 << 10,
+		ALL_BRANCHES_F  = 1 << 11,
+		WHOLE_HISTORY_F = 1 << 12,
+		RANGE_SELECT_F  = 1 << 13,
+		REOPEN_REPO_F   = 1 << 14,
+		USE_CMT_MSG_F   = 1 << 15
 	};
-	const int FLAGS_DEF = 8512;
+	const int FLAGS_DEF = USE_CMT_MSG_F | RANGE_SELECT_F | SMART_LBL_F | VERIFY_CMT_F | SIGN_PATCH_F | LOG_DIFF_TAB_F | MSG_ON_NEW_F;
+
+	// ShaString helpers
+	const ShaString toTempSha(const QString&); // use as argument only, see definition
+	const ShaString toPersistentSha(const QString&, QVector<QByteArray>&);
 
 	// settings helpers
-	uint flags(SCRef group = "");
-	bool testFlag(uint f, SCRef group = "");
-	void setFlag(uint f, bool b, SCRef group = "");
-	void writeSetting(SCRef key, SCRef value, SCRef group = "");
+	uint flags(SCRef flagsVariable);
+	bool testFlag(uint f, SCRef fv = FLAGS_KEY);
+	void setFlag(uint f, bool b, SCRef fv = FLAGS_KEY);
+
+	// tree view icons helpers
+	void initMimePix();
+	void freeMimePix();
+	const QPixmap* mimePix(SCRef fileName);
+
+	// geometry settings helers
+	typedef QVector<QSplitter*> splitVect;
+	void saveGeometrySetting(SCRef name, QWidget* w = NULL, splitVect* svPtr = NULL);
+	void restoreGeometrySetting(SCRef name, QWidget* w = NULL, splitVect* svPtr = NULL);
 
 	// misc helpers
 	bool stripPartialParaghraps(const QByteArray& src, QString* dst, QString* prev);
 	bool writeToFile(SCRef fileName, SCRef data, bool setExecutable = false);
 	bool writeToFile(SCRef fileName, const QByteArray& data, bool setExecutable = false);
 	bool readFromFile(SCRef fileName, QString& data);
-	void baAppend(QByteArray** src, const char* ascii, int len);
-	void baAppend(QByteArray& ba, const QByteArray& data);
+	bool startProcess(QProcess* proc, SCList args, SCRef buf = "", bool* winShell = NULL);
 
 	// cache file
 	const uint C_MAGIC  = 0xA0B0C0D0;
-	const int C_VERSION = 13;
+	const int C_VERSION = 15;
 
 	extern const QString BAK_EXT;
 	extern const QString C_DAT_FILE;
 
 	// misc
 	const int MAX_DICT_SIZE    = 100003; // must be a prime number see QDict docs
-	const int MAX_MENU_ENTRIES = 15;
+	const int MAX_MENU_ENTRIES = 20;
 	const int MAX_RECENT_REPOS = 7;
 	extern const QString QUOTE_CHAR;
+	extern const QString SCRIPT_EXT;
 }
 
+class ShaString : public QLatin1String {
+public:
+	inline ShaString() : QLatin1String(NULL) {}
+	inline ShaString(const ShaString& sha) : QLatin1String(sha.latin1()) {}
+	inline explicit ShaString(const char* sha) : QLatin1String(sha) {}
+
+	inline bool operator!=(const ShaString& o) const { return !operator==(o); }
+	inline bool operator==(const ShaString& o) const {
+
+		return (latin1() == o.latin1()) || !qstrcmp(latin1(), o.latin1());
+	}
+};
+
 class Rev {
 	// prevent implicit C++ compiler defaults
 	Rev();
 	Rev(const Rev&);
 	Rev& operator=(const Rev&);
 public:
-	Rev(const QByteArray& b, uint s, int idx, int* next)
+	Rev(const QByteArray& b, uint s, int idx, int* next, bool withDiff)
 	    : orderIdx(idx), ba(b), start(s) {
 
-		isDiffCache = isApplied = isUnApplied = false;
+		indexed = isDiffCache = isApplied = isUnApplied = false;
 		descRefsMaster = ancRefsMaster = descBrnMaster = -1;
-		*next = indexData();
+		*next = indexData(true, withDiff);
 	}
-	bool isBoundary() const { return boundary; }
+	bool isBoundary() const { return (ba.at(shaStart - 1) == '-'); }
 	uint parentsCount() const { return parentsCnt; }
-	const QString parent(int idx) const;
+	const ShaString parent(int idx) const;
 	const QStringList parents() const;
-	const QString sha() const { return mid(start + startOfs, 40); }
-	const QString author() const { return mid(autStart, autLen); }
-	const QString authorDate() const { return mid(autDateStart, autDateLen); }
-	const QString shortLog() const { return mid(sLogStart, sLogLen); }
-	const QString longLog() const { return mid(lLogStart, lLogLen); }
+	const ShaString sha() const { return ShaString(ba.constData() + shaStart); }
+	const QString committer() const { setup(); return mid(comStart, autStart - comStart - 1); }
+	const QString author() const { setup(); return mid(autStart, autDateStart - autStart - 1); }
+	const QString authorDate() const { setup(); return mid(autDateStart, 10); }
+	const QString shortLog() const { setup(); return mid(sLogStart, sLogLen); }
+	const QString longLog() const { setup(); return mid(lLogStart, lLogLen); }
+	const QString diff() const { setup(); return mid(diffStart, diffLen); }
 
-	QValueVector<int> lanes, childs;
-	bool isDiffCache, isApplied, isUnApplied;
-	int orderIdx;
-	QValueVector<int> descRefs;     // list of descendant refs index, normally tags
-	QValueVector<int> ancRefs;      // list of ancestor refs index, normally tags
-	QValueVector<int> descBranches; // list of descendant branches index
+	QVector<int> lanes, childs;
+	QVector<int> descRefs;     // list of descendant refs index, normally tags
+	QVector<int> ancRefs;      // list of ancestor refs index, normally tags
+	QVector<int> descBranches; // list of descendant branches index
 	int descRefsMaster; // in case of many Rev have the same descRefs, ancRefs or
 	int ancRefsMaster;  // descBranches these are stored only once in a Rev pointed
 	int descBrnMaster;  // by corresponding index xxxMaster
-
+	int orderIdx;
 private:
-	int indexData();
+	inline void setup() const { if (!indexed) indexData(false, false); }
+	int indexData(bool quick, bool withDiff) const;
 	const QString mid(int start, int len) const;
+	const QString midSha(int start, int len) const;
 
 	const QByteArray& ba; // reference here!
-	const uint start;
-	bool boundary;
-	uint parentsCnt, startOfs;
-	int autStart, autLen, autDateStart, autDateLen;
-	int sLogStart, sLogLen, lLogStart, lLogLen;
+	const int start;
+	mutable int parentsCnt, shaStart, comStart, autStart, autDateStart;
+	mutable int sLogStart, sLogLen, lLogStart, lLogLen, diffStart, diffLen;
+	mutable bool indexed;
+public:
+	bool isDiffCache, isApplied, isUnApplied; // put here to optimize padding
 };
-typedef QDict<Rev> RevMap; // faster then a map
+typedef QHash<ShaString, const Rev*> RevMap;  // faster then a map
+
 
 class RevFile {
 
@@ -316,13 +374,14 @@ class RevFile {
 	// When status of all the files is 'modified' then onlyModified is
 	// set, this let us to do some optimization in this common case
 	bool onlyModified;
-	QValueVector<int> status;
-	QValueVector<QString> extStatus;
+	QVector<int> status;
+	QVector<QString> extStatus;
 
 	// prevent implicit C++ compiler defaults
 	RevFile(const RevFile&);
 	RevFile& operator=(const RevFile&);
 public:
+
 	enum StatusFlag {
 		MODIFIED = 1,
 		DELETED  = 2,
@@ -335,17 +394,31 @@ public:
 	};
 
 	RevFile() : onlyModified(true) {}
-	QValueVector<int> dirs; // index of a string vector
-	QValueVector<int> names;
-	QValueVector<int> mergeParent;
+
+	/* This QByteArray keeps indices in some dir and names vectors,
+	 * defined outside RevFile. Paths are splitted in dir and file
+	 * name, first all the dirs are listed then the file names to
+	 * achieve a better compression when saved to disk.
+	 * A single QByteArray is used instead of two vectors because it's
+	 * much faster to load from disk when using a QDataStream
+	 */
+	QByteArray pathsIdx;
+
+	int dirAt(uint idx) const { return ((int*)pathsIdx.constData())[idx]; }
+	int nameAt(uint idx) const { return ((int*)pathsIdx.constData())[count() + idx]; }
+
+	QVector<int> mergeParent;
 
 	// helper functions
-	int count() const { return dirs.count(); }
+	int count() const {
+
+		return pathsIdx.size() / ((int)sizeof(int) * 2);
+	}
 	bool statusCmp(int idx, StatusFlag sf) const {
 
 		return ((onlyModified ? MODIFIED : status.at(idx)) & sf);
 	}
-	const QString extendedStatus(uint idx) const {
+	const QString extendedStatus(int idx) const {
 	/*
 	   rf.extStatus has size equal to position of latest copied/renamed file,
 	   that could be lower then count(), so we have to explicitly check for
@@ -353,8 +426,11 @@ public:
 	*/
 		return (!extStatus.isEmpty() && idx < extStatus.count() ? extStatus.at(idx) : "");
 	}
+	const RevFile& operator>>(QDataStream&) const;
+	RevFile& operator<<(QDataStream&);
 };
-typedef QDict<RevFile> RevFileMap;
+typedef QHash<ShaString, const RevFile*> RevFileMap;
+
 
 class FileAnnotation {
 public:
@@ -365,10 +441,12 @@ public:
 	int annId;
 	QString fileSha;
 };
+typedef QHash<ShaString, FileAnnotation> AnnotateHistory;
 
-class BaseEvent: public QCustomEvent {
+
+class BaseEvent: public QEvent {
 public:
-	BaseEvent(SCRef d, int id) : QCustomEvent(id), payLoad(d) {}
+	BaseEvent(SCRef d, int id) : QEvent((QEvent::Type)id), payLoad(d) {}
 	const QString myData() const { return payLoad; }
 private:
 	const QString payLoad; // passed by copy
