@@ -1,5 +1,5 @@
 /*
-	Author: Marco Costalba (C) 2005-2006
+	Author: Marco Costalba (C) 2005-2007
 
 	Copyright: See COPYING file that comes with this distribution
 
@@ -7,73 +7,94 @@
 #ifndef MAINIMPL_H
 #define MAINIMPL_H
 
-#include <qguardedptr.h>
-#include <qregexp.h>
-#include <qprocess.h>
-#include <qptrvector.h>
-#include <qtimer.h>
+#include <QProcess>
+#include <QRegExp>
+#include <QDir>
 #include "exceptionmanager.h"
 #include "common.h"
-#include "mainbase.h"
+#include "ui_mainview.h"
 
-class QListViewItem;
-class QListBoxItem;
-class QPopupMenu;
-class QCustomEvent;
-class QAccel;
+class QAction;
+class QCloseEvent;
+class QComboBox;
+class QEvent;
+class QListWidgetItem;
+class QModelIndex;
+class QProgressBar;
+class QShortcutEvent;
 class QTextEdit;
-class Git;
-class Domain;
-class ListViewItem;
-class RevsView;
-class FileView;
-class FileHistory;
 
-class MainImpl : public MainBase {
+class Domain;
+class Git;
+class FileHistory;
+class FileView;
+class RevsView;
+
+class MainImpl : public QMainWindow, public Ui_MainBase {
 Q_OBJECT
 public:
 	MainImpl(const QString& curDir = "", QWidget* parent = 0);
-	QString curWorkDir() const { return curDir; }
+	void updateContextActions(SCRef newRevSha, SCRef newFileName, bool isDir, bool found);
+	const QString getRevisionDesc(SCRef sha);
+
+	// not buildable with Qt designer, will be created manually
+	QLineEdit* lineEditSHA;
+	QLineEdit* lineEditFilter;
+
+	enum ComboSearch {
+		CS_SHORT_LOG,
+		CS_LOG_MSG,
+		CS_AUTHOR,
+		CS_SHA1,
+		CS_FILE,
+		CS_PATCH,
+		CS_PATCH_REGEXP
+	};
+
+	QComboBox* cmbSearch;
 
 signals:
 	void highlightPatch(const QString&, bool);
 	void updateRevDesc();
 	void closeAllWindows();
 	void closeAllTabs();
-	void tabClosed(int tabPos);
-	void repaintListViews(const QFont&);
+	void changeFont(const QFont&);
 	void closeTabButtonEnabled(bool);
+	void typeWriterFontChanged();
+	void flagChanged(uint);
 
-public slots:
-	void tabWdg_currentChanged(QWidget*);
-	void on_newRevsAdded(const FileHistory*, const QValueVector<QString>&);
+private slots:
+	void tabWdg_currentChanged(int);
+	void newRevsAdded(const FileHistory*, const QVector<ShaString>&);
+	void fileNamesLoad(int, int);
+	void revisionsDragged(const QStringList&);
+	void revisionsDropped(const QStringList&);
+	void shortCutActivated();
 
 protected:
-	virtual void customEvent(QCustomEvent* e);
+	virtual bool event(QEvent* e);
 
 protected slots:
 	void initWithEventLoopActive();
 	void refreshRepo(bool setCurRevAfterLoad = true);
-	void on_listViewLog_doubleClicked(QListViewItem*);
-	void on_fileList_doubleClicked(QListBoxItem*);
-	void on_treeView_doubleClicked(QListViewItem*);
-	void on_histListView_doubleClicked(QListViewItem* item);
-	void on_customActionListChanged(const QStringList& list);
-	void on_openRecent_activated(int id);
-	void on_customAction_activated(int id);
-	void on_customAction_exited(const QString& name);
-	void on_goRef_activated(int id);
-	void on_changesCommitted(bool);
+	void listViewLog_doubleClicked(const QModelIndex&);
+	void fileList_itemDoubleClicked(QListWidgetItem*);
+	void treeView_doubleClicked(QTreeWidgetItem*, int);
+	void histListView_doubleClicked(const QModelIndex&);
+	void customActionListChanged(const QStringList& list);
+	void openRecent_triggered(QAction*);
+	void customAction_triggered(QAction*);
+	void customAction_exited(const QString& name);
+	void goRef_triggered(QAction*);
+	void changesCommitted(bool);
 	void lineEditSHA_returnPressed();
 	void lineEditFilter_returnPressed();
 	void pushButtonCloseTab_clicked();
-	void toolButtonFilter_toggled(bool);
-	void toolButtonBold_toggled(bool);
-	void accelActivated(int);
 	void ActBack_activated();
 	void ActForward_activated();
 	void ActFind_activated();
 	void ActFindNext_activated();
+	void ActRangeDlg_activated();
 	void ActViewRev_activated();
 	void ActViewFile_activated();
 	void ActViewFileNewTab_activated();
@@ -81,7 +102,8 @@ protected slots:
 	void ActViewDiffNewTab_activated();
 	void ActExternalDiff_activated();
 	void ActSplitView_activated();
-	void ActShowDescHeader_toggled(bool);
+	void ActToggleLogsDiff_activated();
+	void ActShowDescHeader_activated();
 	void ActOpenRepo_activated();
 	void ActOpenRepoNewWindow_activated();
 	void ActRefresh_activated();
@@ -90,12 +112,16 @@ protected slots:
 	void ActMailApplyPatch_activated();
 	void ActSettings_activated();
 	void ActCommit_activated();
+	void ActAmend_activated();
+	void ActBranch_activated();
 	void ActTag_activated();
 	void ActTagDelete_activated();
 	void ActPush_activated();
 	void ActPop_activated();
 	void ActClose_activated();
 	void ActExit_activated();
+	void ActSearchAndFilter_toggled(bool);
+	void ActSearchAndHighlight_toggled(bool);
 	void ActCustomActionSetup_activated();
 	void ActCheckWorkDir_toggled(bool);
 	void ActShowTree_toggled(bool);
@@ -105,106 +131,82 @@ protected slots:
 	void closeEvent(QCloseEvent* ce);
 
 private:
-	friend class FileView;
-	friend class RevsView;
-	friend class PatchView;
 	friend class setRepoDelayed;
 
 	virtual bool eventFilter(QObject* obj, QEvent* ev);
 	void updateGlobalActions(bool b);
-	void updateContextActions(SCRef newRevSha, SCRef newFileName, bool isDir, bool found);
-	void setupAccelerator(QAccel* accel);
+	void setupShortcuts();
 	int currentTabType(Domain** t);
 	void filterList(bool isOn, bool onlyHighlight);
-	bool passFilter(ListViewItem* i, const QRegExp& f, int cn, const QMap<QString,bool>& sm);
-	void setRepository(SCRef wd, bool r, bool ks, QStringList* fl = NULL);
-	void diffViewerUpdate(QListViewItem* item, bool newDiff);
-	void getExternalDiffArgs(QStringList* args);
+	bool isMatch(SCRef sha, SCRef f, int cn, const QMap<QString,bool>& sm);
+	void highlightAbbrevSha(SCRef abbrevSha);
+	void setRepository(SCRef wd, bool = false, bool = false, const QStringList* = NULL, bool = false);
+	void getExternalDiffArgs(QStringList* args, QStringList* filenames);
 	void lineEditSHASetText(SCRef text);
 	void updateCommitMenu(bool isStGITStack);
 	void updateRecentRepoMenu(SCRef newEntry = "");
 	void doUpdateRecentRepoMenu(SCRef newEntry);
 	void doUpdateCustomActionMenu(const QStringList& list);
+	void doBranchOrTag(bool isTag);
 	void ActCommit_setEnabled(bool b);
 	void doContexPopup(SCRef sha);
 	void doFileContexPopup(SCRef fileName, int type);
 	void adjustFontSize(int delta);
 	void scrollTextEdit(int delta);
-	void scrollListView(int delta);
 	void goMatch(int delta);
 	bool askApplyPatchParameters(bool* commit, bool* fold);
+	void saveCurrentGeometry();
 	QTextEdit* getCurrentTextEdit();
-	template<class X> QPtrList<X>* getTabs(int tabPos = -1);
-	template<class X> X* firstTab(int startPos = -1);
+	template<class X> QList<X*>* getTabs(QWidget* tabPage = NULL);
+	template<class X> X* firstTab(QWidget* startPage = NULL);
 	void openFileTab(FileView* fv = NULL);
 
 	EM_DECLARE(exExiting);
 
 	Git* git;
 	RevsView* rv;
+	QProgressBar* pbFileNamesLoading;
 
 	// curDir is the repository working dir, could be different from qgit running
 	// directory QDir::current(). Note that qgit could be run from subdirectory
 	// so only after git->isArchive() that updates curDir to point to working dir
 	// we are sure is correct.
 	QString curDir;
-	QRegExp shortLogRE;
-	QRegExp longLogRE;
-	QGuardedPtr<QPopupMenu> contextMenu;
-	QGuardedPtr<QPopupMenu> contextSubMenu;
-	QGuardedPtr<QPopupMenu> contextRmtMenu;
 	QString startUpDir;
 	QString textToFind;
-	QFont listViewFont;
+	QRegExp shortLogRE;
+	QRegExp longLogRE;
 	bool setRepositoryBusy;
-	int recentRepoMenuPos;
 };
 
 class ExternalDiffProc : public QProcess {
 Q_OBJECT
 public:
-	ExternalDiffProc(const QStringList& args, QObject* p) : QProcess(args, p) {
+	ExternalDiffProc(const QStringList& f, QObject* p)
+		: QProcess(p), filenames(f) {
 
-		connect(this, SIGNAL(processExited()), this, SLOT(on_processExited()));
+		connect(this, SIGNAL(finished(int, QProcess::ExitStatus)),
+		        this, SLOT(on_finished(int, QProcess::ExitStatus)));
 	}
 	~ExternalDiffProc() {
 
-		tryTerminate();
+		terminate();
 		removeFiles();
 	}
+	QStringList filenames;
+
 private slots:
-	void on_processExited() { deleteLater(); }
+	void on_finished(int, QProcess::ExitStatus) { deleteLater(); }
 
 private:
 	void removeFiles() {
 
-		if (!arguments().empty()) {
+		if (!filenames.empty()) {
 			QDir d; // remove temporary files to diff on
-			d.remove(arguments()[1]);
-			d.remove(arguments()[2]);
+			d.remove(filenames[0]);
+			d.remove(filenames[1]);
 		}
 	}
 };
 
-class setRepoDelayed : public QObject {
-Q_OBJECT
-public:
-	setRepoDelayed(MainImpl* mi, SCRef nd, bool r, bool ks, QStringList* fl) :
-	QObject(mi), m(mi), newDir(nd), refresh(r), keepSelection(ks), filterList(fl) {
-
-		QTimer::singleShot(100, this, SLOT(on_timeout()));
-	}
-private slots:
-	void on_timeout() {
-
-		m->setRepository(newDir, refresh, keepSelection, filterList);
-		deleteLater();
-	 }
-private:
-	MainImpl* m;
-	const QString newDir;
-	bool refresh, keepSelection;
-	QStringList* filterList;
-};
-
 #endif
