@@ -1,5 +1,5 @@
 /*
-	Author: Marco Costalba (C) 2005-2006
+	Author: Marco Costalba (C) 2005-2007
 
 	Copyright: See COPYING file that comes with this distribution
 
@@ -7,14 +7,14 @@
 #ifndef RANGESELECTIMPL_H
 #define RANGESELECTIMPL_H
 
-#include "rangeselectbase.h"
+#include "ui_rangeselect.h"
 
 class Git;
 
-class RangeSelectImpl: public RangeSelectBase {
+class RangeSelectImpl: public QDialog, public Ui_RangeSelectBase {
 Q_OBJECT
 public:
-	RangeSelectImpl(QWidget* par, QString* range, const QStringList& tl, bool* quit, Git* g);
+	RangeSelectImpl(QWidget* par, QString* range, bool rc, Git* g);
 
 public slots:
 	void pushButtonOk_clicked();
@@ -24,13 +24,10 @@ public slots:
 	void checkBoxShowWholeHistory_toggled(bool b);
 
 private:
-	void orderTags(const QStringList& src, QStringList& dst);
+	void orderRefs(const QStringList& src, QStringList& dst);
 
-	QString* range;
-	bool* quit;
 	Git* git;
-	QString fromBckUp;
-	QString toBckUp;
+	QString* range;
 };
 
 #endif
